#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <assert.h>
#include <string.h>
#include <math.h>

#include <vector>
#include <queue>
#include <utility>

#define dprintf nil
inline void nil(...){}

#define ROUND(a) (int(floor(a+0.5f)))

/****************************************************************************************************************
 *
 */
class rgb
{
	public:
	rgb()
	{
		r = g = b = 0;
	}
	rgb(int pr,int pg,int pb)
	{
		r = pr;g = pg; b = pb;
	}
	rgb(const rgb &RHS)
	{
		r = RHS.r;g = RHS.g;b = RHS.b;
	}
	rgb operator =(const rgb RHS)
	{
		r = RHS.r;g = RHS.g; b = RHS.b;
	}
	
//  Transormation operatons
	static int dot(const rgb L,const rgb R)
	{
		return L.r * R.r + L.g * R.g + L.b * R.b;
	}
	static int gauss(const rgb L,const rgb R)
	{
		return int(255.0 * exp(- delta(L,R) / (768.0 * 2)));
	}
	static int delta(const rgb L,const rgb R)
	{
		return abs(L.r-R.r) + abs(L.g - R.g) + abs(L.b - R.b);
	}
	static float relative(const rgb L,float *ratio)
	{
		float pixel[3];
		pixel[0] = L.r/255.0f;
		pixel[1] = L.g/255.0f;
		pixel[2] = L.b/255.0f;
		assert(ratio[0] * ratio[1] * ratio[2] != 0.0f);
		pixel[0] /= ratio[0];
		pixel[1] = (pixel[1] / ratio[1]) / pixel[0];
		pixel[2] = (pixel[2] / ratio[2]) / pixel[0];
		return fabs(pixel[1]-1.0f)+fabs(pixel[2]-1.0f);
		
	}

//more utitlities and members	
	void grey(int val)
	{
		r = g = b = val;
	}
	float ave()
	{
		return (r+g+b)/3.0f;
	}
	unsigned char r;
	unsigned char g;
	unsigned char b;
};

/****************************************************************************************************************
 *
 */
class ppm
{
public:
	ppm()
	{
		width = height = depth = 0;
		data = NULL;
	}
	
	ppm(const ppm &RHS)
	{
		width = RHS.width;
		height = RHS.height;
		depth = RHS.depth;
		data =  new rgb[width * height];
		for(int i = 0; i < width*height; i++)
			data[i] = RHS.data[i];
	}
	
	ppm operator=(const ppm &RHS)
	{
		if(data)
			delete [] data;
		width = RHS.width;
		height = RHS.height;
		depth = RHS.depth;
		data =  new rgb[width * height];
		for(int i = 0; i < width*height; i++)
			data[i] = RHS.data[i];
		return *this;
	}
	
	rgb &get(int x,int y)
	{
		assert(x >= 0);
		assert(y >= 0);
		assert(x < width);
		assert(y < height);
		return data[width * y + x];
	}
	
	bool load(const char *fname)
	{
		int junk;
		char trash[128];
		FILE *source;
		size_t count;
		dprintf("+----------------\n"
				"| Loading file\n"
				"| Attempting to open %s\n",fname);
		source = fopen(fname,"rb");
		
		assert(source);
		assert(!ferror(source));
		assert(!feof(source));
		
		fscanf(source,"P6 ");
		while((junk = fgetc(source)) == '#')
		{
			fgets(trash,128,source);
		}
		ungetc(junk,source);
		fscanf(source,"%i %i %i\n",&width,&height,&depth);
		data = new rgb[width * height];
		count = fread(data,sizeof(unsigned char),width*height*3,source);
		dprintf("| read in %i bytes\n",count);
		
		fclose(source);
		dprintf("\n\n");
		
		return true;
	}
	
	void write(const char *fname)
	{
		dprintf("+----------------\n"
		        "| Writting file\n"
				"| outfile file is %s\n",fname);
		FILE *stream = fopen(fname,"wb");
		assert(stream);
		fprintf(stream,"P6\n%i %i\n%i\n",width,height,depth);
		fwrite(data,width*height*3,sizeof(unsigned char),stream);
		fclose(stream);
		dprintf("| wrote %i bytes\n",width*height*3);
		dprintf("\n\n");
	}

	void threshold(int min)
	{
		int i;
		for(i = 0; i < width * height; i++)
		{
			if(data[i].ave() < min){
				data[i].grey(0);
			}else{
				data[i].grey(255);
			}
		}
	}
	
	void blank(int pWidth,int pHeight)
	{
		int i;
		width = pWidth;
		height = pHeight;
		depth = 255;
		data = new rgb[width * height];
		for(i = 0; i < width*height; i++)
		{
			data[i].grey(0);
		}
	}
	
	void blank(const ppm &R)
	{
		blank(R.width,R.height);
	}

	int count()
	{
		int total = 0;
		for(int i = 0; i < width*height; i++)
			if(data[i].ave() > 0)
				total++;
		return total;
	}

	bool valid(int x,int y)
	{
		return (x >= 0 && x < width && y >= 0 && y < height);
	}

	static void mask(ppm &dest,ppm &source, ppm &mask)
	{
		int i;
		if(dest.data == NULL)
			dest.blank(source.width,source.height);
			
		for(i = 0; i < source.width*source.height; i++)
		{
			if(mask.data[i].ave() > 128)
				dest.data[i] = source.data[i];
			else
				dest.data[i].grey(0);
		}
	}

	static ppm intersect(ppm &L, ppm &R)
	{
		ppm temp;
		assert(L.width == R.width);
		assert(L.height == R.height);
		temp.blank(L);
		for(int i = 0; i < L.width * L.height; i++)
			temp.data[i].grey(    (L.data[i].ave() > 0 && R.data[i].ave() > 0) ? 255 : 0);
		return temp;
	}

	void drawCircle(float radius,int cx,int cy,rgb color = rgb(255,255,255))
	{
		float i;
		int x,y;
		for(i = 0.0f; i < 2 * M_PI; i += 1/radius){
			x = ROUND(cx + radius * cos(i));
			y = ROUND(cy + radius * sin(i));
			if(valid(x,y)){
				get(x,y) = color;
			}
		}
	}

	void drawCircle(float radius,std::pair<int,int> center,rgb color = rgb(255,255,255))
	{
		drawCircle(radius,center.first,center.second,color);
	}
	
	int width;
	int height;
	int depth;
	rgb *data;
};

/****************************************************************************************************************
 *
 */
struct Params{
	Params(){
		thresh = 230;
		memset(infile,0,128);
		memset(outfile,0,128);
		memset(maskfile,0,128);
		
		color.r = 255;
		color.g = 255;
		color.b = 255;
	}
	void load(int argc,char **argv)
	{
		int op;
		int flag = 1;
		int i;
		dprintf("+----------------\n"
		        "| Loading options\n");
		for(i = 1; i < argc; i++){
			if(!strcmp("-i",argv[i])){
				strcpy(infile,argv[++i]);
				dprintf("| intput file read in as %s\n",argv[i]);
			}else if(!strcmp("-o",argv[i])){
				strcpy(outfile,argv[++i]);
				dprintf("| output file read in as %s\n",argv[i]);
			}else if(!strcmp("-c",argv[i])){
				sscanf(argv[++i],"%i,%i,%i",&(color.r),&(color.g),&(color.b));
				dprintf("| color read as %s\n",argv[i]);
			}else if(!strcmp("-r",argv[i])){
				sscanf(argv[++i],"%f,%f,%f",ratio,ratio+1,ratio+2);
				dprintf("| ratio read as %f,%f,%f\n",ratio[0],ratio[1],ratio[2]);			
			}else if(!strcmp("-t",argv[i])){
				sscanf(argv[++i],"%i",&thresh);
				dprintf("| read in threshold value as %i\n",thresh);
			}else if(!strcmp("-v",argv[i])){
				strcpy(overlayfile,argv[++i]);
				dprintf("| overlay file read in as %s\n",overlayfile);			
			}else if(!strcmp("-m",argv[i])){
				strcpy(maskfile,argv[++i]);
				dprintf("| mask output file read as %s\n",maskfile);
			}else if(!strcmp("-h",argv[i])){
				printf("usage:\n"
				       "./poc -i <infile> -c #R,#G,#B [-o <outfile>] [-m <maskfile>] [-t <threshold>]\n");
				exit(0);
			
			}else{
				printf("!> Error: Unknown arg \'%s\'\n",argv[i]);
			}
		}
		dprintf("\n\n");
		if(infile[0] == 0){
			printf("!> Error: No input file specified\n");
			exit(0);
		}
	}
	int thresh;
	char infile[128];
	char outfile[128];
	char overlayfile[128];
	char maskfile[128];
	float ratio[3];
	rgb color;
} params;

/****************************************************************************************************************
 *
 */
class region : public std::vector< std::pair<int,int> >
{
public:
	region():std::vector< std::pair<int,int> >(){}
	virtual ~region(){}
	int fill(ppm &RHS,int px,int py)
	{
		char cursor[] = {'|','/','-','\\'};int cindex = 0;
		int x,y,x0,y0;
		std::queue<std::pair<int,int> > seen;
		assert(RHS.get(px,py).ave() > 127);
		seen.push(std::pair<int,int>(px,py));
//		dprintf("%c",cursor[cindex = (cindex+1)%4]);fflush(stdout);
		while(!seen.empty()){
//			dprintf("\b%c",cursor[cindex = (cindex+1)%4]);fflush(stdout);
			
			x0 = seen.front().first;
			y0 = seen.front().second;
			RHS.get(x0,y0).grey(0);
			
			//layer1
			x = x0-1; y = y0-1;
			if(RHS.valid(x,y) && RHS.get(x,y).ave() > 127){
				seen.push(std::pair<int,int>(x,y));
				RHS.get(x,y).grey(0);
			}
			x = x0; y = y0-1;
			if(RHS.valid(x,y) && RHS.get(x,y).ave() > 127){
				seen.push(std::pair<int,int>(x,y));
				RHS.get(x,y).grey(0);
			}
			x = x0+1; y = y0-1;
			if(RHS.valid(x,y) && RHS.get(x,y).ave() > 127){
				seen.push(std::pair<int,int>(x,y));
				RHS.get(x,y).grey(0);
			}
			//layer2
			x = x0-1; y = y0;
			if(RHS.valid(x,y) && RHS.get(x,y).ave() > 127){
				seen.push(std::pair<int,int>(x,y));
				RHS.get(x,y).grey(0);
			}
			x = x0+1; y = y0;
			if(RHS.valid(x,y) && RHS.get(x,y).ave() > 127){
				seen.push(std::pair<int,int>(x,y));
				RHS.get(x,y).grey(0);
			}
			//layer3
			x = x0-1; y = y0+1;
			if(RHS.valid(x,y) && RHS.get(x,y).ave() > 127){
				seen.push(std::pair<int,int>(x,y));
				RHS.get(x,y).grey(0);
			}
			x = x0; y = y0+1;
			if(RHS.valid(x,y) && RHS.get(x,y).ave() > 127){
				seen.push(std::pair<int,int>(x,y));
				RHS.get(x,y).grey(0);
			}
			x = x0+1; y = y0+1;
			if(RHS.valid(x,y) && RHS.get(x,y).ave() > 127){
				seen.push(std::pair<int,int>(x,y));
				RHS.get(x,y).grey(0);
			}
			push_back(seen.front());
			seen.pop();
		}

		return (int)size();
	}
	int area()
	{
		int minx,miny,maxx,maxy;
		minx = maxx = front().first;
		miny = maxy = front().second;
		for(std::vector<std::pair<int,int> >::iterator i = begin(); i != end(); i++){
			maxx >?= i->first;
			minx <?= i->first;
			maxy >?= i->second;
			miny <?= i->second;
		}
		return (maxx-minx+1)*(maxy-miny+1);
	}
	std::pair<int,int> center()
	{
		int x = 0;
		int y = 0;
		for(std::vector<std::pair<int,int> >::iterator i = begin(); i != end(); i++){
			x += i->first;
			y += i->second;
		}
		x /= size();
		y /= size();
		return std::pair<int,int>(x,y);
	}
	float radius()
	{
		std::pair<int,int> dims = dimentions();
		float width = dims.first/2.0f;
		float height = dims.second/2.0f;
		return sqrtf(width*width + height*height);
	}
	bool contains(std::pair<int,int> RHS)
	{
		if(find(RHS) == end())
			return false;
		else
			return true;
	}
	std::vector< std::pair<int,int> >::iterator find(std::pair<int,int> RHS)
	{
		std::vector< std::pair<int,int> >::iterator i; 
		for(i = begin(); i != end() && *i != RHS; i++);
		return i;
	}
	virtual void push_back(std::pair<int,int> it)
	{
		if(!contains(it))
			std::vector< std::pair<int,int> >::push_back(it);
	}
	std::pair<int,int> dimentions()
	{
		int minx,miny,maxx,maxy;
		minx = maxx = front().first;
		miny = maxy = front().second;
		for(std::vector<std::pair<int,int> >::iterator i = begin(); i != end(); i++){
			maxx >?= i->first;
			minx <?= i->first;
			maxy >?= i->second;
			miny <?= i->second;
		}
		return std::pair<int,int>((maxx-minx+1),(maxy-miny+1));
	}
	float slope()
	{
		std::pair<int,int> dim = dimentions();
		return (dim.first/float(dim.second));
	}
};

/****************************************************************************************************************
 *
 */
class region_list : public std::vector<region>
{
public:
	region_list():std::vector<region>(){}
	virtual ~region_list(){}
	void fill(ppm &RHS)
	{
		int num;
		ppm tmp = RHS;
		region cur;
		int i,j;
		dprintf("+----------------\n"
		        "| Scanning regions\n"
				"| Total count = %i\n",tmp.count());
		for(i = 1; i < tmp.width-1; i++){
			for(j = 1; j < tmp.height-1; j++){
				if(tmp.get(i,j).ave() > 0){
					num = cur.fill(tmp,i,j);
					if(num > 40)
						dprintf("| Region %u with %i pixels\n",size(),num);
					push_back(cur);
					cur.clear();
				}
			}
		}
		dprintf("\n\n");
	}
	void reduce()
	{
		for(region_list::iterator i = begin(); i != end(); i++){
			if(i->size() < 40){
				erase(i);
				--i;
			}else if(i->size() == i->area()){
				erase(i);
				--i;
			}else if(i->area()/float(i->size()) > 2){
				erase(i);
				--i;
			}else if(fabs(i->slope() - 0.4f) > 0.2f){
				erase(i);
				--i;			
			}
		}	
	}
	ppm mask(ppm &RHS,char *fname = 0)
	{
		int code,num;
		ppm temp;
		temp.blank(RHS);
		
		dprintf("+----------------\n"
				"| Creating mask from region list\n");
		num = 0;
		for(region_list::iterator i = begin(); i != end(); i++){
			++num;
			dprintf("| Region: %2i    Grey: %i\n",num,(255*num)/(int)size());
			for(region::iterator j = i->begin(); j != i->end();j++){
				temp.get(j->first,j->second).grey((255 * num)/(int)size());
//				dprintf("| Setting pixel:(%i,%i)\n",j->first,j->second);
			}
		}
		dprintf("| Masked image: %i pixels\n\n\n",temp.count());
		if(fname)
			temp.write(fname);
		return temp;
	}
	void dump()
	{	
		printf("+----------------\n"
				"| Analyzing image\n");
		for(region_list::iterator i = begin(); i != end(); i++){
			printf("| Area: %5i    Size: %5u    Ratio: %.3f    Slope: %.3f\n",i->area(),i->size(),i->area()/float(i->size()),i->slope());
		}
		printf("| Total regions: %u\n",size());
		printf("\n\n");
	}
};

/****************************************************************************************************************
 *
 */
int main(int argc,char **argv)
{
	int i;
	ppm img;
	ppm mask;
	ppm mask2;
	ppm out;
	ppm tmpMask;
	ppm out2;
	ppm out3;

	region_list blah;
	
	params.load(argc,argv);
	img.load(params.infile);
	
	out.blank(img);
	mask.blank(img);
	mask2.blank(img);

	//The actual image processing goes here
	for(i = 0; i < img.width*img.height; i++)\
		mask.data[i].grey(rgb::gauss(img.data[i],params.color));
	
	for(i = 0; i < img.width*img.height; i++)\
		mask2.data[i].grey((rgb::relative(img.data[i],params.ratio)) > 0.8f ? 0 : 255);
	if(params.thresh >= 0)
		mask.threshold(params.thresh);
	
	
	ppm::mask(out,img,mask);
	
	

	out3 = ppm::intersect(mask,mask2);

	blah.fill(out3);
	blah.reduce();
	
	out2 = blah.mask(img);

	if(blah.size())
	{
		for(region_list::iterator i = blah.begin(); i != blah.end(); i++)
			img.drawCircle(i->radius(),i->center(),rgb(0,0,0));
		printf("Detected: %i cans\n",blah.size());
		
		img.write("scan.ppm");
		out.write("out1.ppm");
		mask.write("mask1.ppm");
		mask2.write("mask2.ppm");

		
	}
	return 0;
}

